; The following is Identify Cpu by MichaelW

    ; -------------------------------------------
    ; Returns a bit range value, with the value
    ; shifted to place lowbit at bit position 0.
    ; -------------------------------------------
    brval MACRO dwordval, highbit, lowbit
        mov   eax, dwordval
        shl   eax, 31-highbit
        shr   eax, 31-highbit+lowbit
        EXITM <eax>
    ENDM

    ; --------------------------------------------
    ; This will overwrite EAX, EBX, ECX, and EDX.
    ; --------------------------------------------
    _cpuid MACRO function
        mov   eax, function
        cpuid
    ENDM

    ; --------------------------------------
    ; Returns 1 if the specified feature is
    ; supported, otherwise 0.
    ; --------------------------------------
    cpufeat MACRO feature:REQ
        push  ebx
        IFIDNI <feature>, <CPUID>
            ;; ---------------------------------
            ;; CPUID supported if can set/clear
            ;; ID flag (EFLAGS bit 21).
            ;; ---------------------------------
            pushfd
            pop   edx
            pushfd
            pop   eax
            xor   eax, 200000h  ; flip ID flag
            push  eax
            popfd
            pushfd
            pop   eax
            xor   eax, edx
        ELSEIFIDNI <feature>, <FPU>
            _cpuid 1
            mov   eax, brval(edx,0,0)
        ELSEIFIDNI <feature>, <TSC>
            _cpuid 1
            mov   eax, brval(edx,4,4)
        ELSEIFIDNI <feature>, <CX8>
            _cpuid 1
            mov   eax, brval(edx,8,8)
        ELSEIFIDNI <feature>, <CMOV>
            _cpuid 1
            mov   eax, brval(edx,15,15)
        ELSEIFIDNI <feature>, <CLFSH>
            _cpuid 1
            mov   eax, brval(edx,19,19)
        ELSEIFIDNI <feature>, <MMX>
            _cpuid 1
            mov   eax, brval(edx,23,23)
        ELSEIFIDNI <feature>, <FXSR>
            _cpuid 1
            mov   eax, brval(edx,24,24)
        ELSEIFIDNI <feature>, <SSE>
            _cpuid 1
            mov   eax, brval(edx,25,25)
        ELSEIFIDNI <feature>, <SSE2>
            _cpuid 1
            mov   eax, brval(edx,26,26)
        ELSEIFIDNI <feature>, <HTT>
            _cpuid 1
            mov   eax, brval(edx,28,28)
        ELSEIFIDNI <feature>, <SSE3>
            _cpuid 1
            mov   eax, brval(ecx,0,0)
        ENDIF        
        pop   ebx
        EXITM <eax>
    ENDM

    VENDOR_OTHER  EQU 0
    VENDOR_INTEL  EQU 1
    VENDOR_AMD    EQU 2

    SWITCH$ EQU <switch$>
    CASE$   EQU <case$>
    ELSE$   EQU <else$>
    ENDSW$  EQU <endsw$>

    NL EQU <13,10>

    .code

IdentifyCPU proc uses ebx esi
    LOCAL maxCpuidInput   :DWORD  ; maximum basic function
    LOCAL signature       :DWORD  ; EAX (function 1)
    LOCAL steppingID      :DWORD  ; EAX bits 3-0 (function 1)
    LOCAL model           :DWORD  ; EAX bits 7-4 (function 1)
    LOCAL displayModel    :DWORD  ; calculated
    LOCAL family          :DWORD  ; EAX bits 11-8 (function 1)
    LOCAL displayFamily   :DWORD  ; calculated
    LOCAL extendedModel   :DWORD  ; EAX bits 19-16 (function 1)
    LOCAL extendedFamily  :DWORD  ; EAX bits 27-20 (function 1)
    LOCAL brandID         :DWORD  ; EBX bits 7-0 (function 1)

    LOCAL maxCpuidInputEx :DWORD  ; maximum extended function
    LOCAL signatureEx     :DWORD  ; EAX (function 80000001h)

    LOCAL vendor          :DWORD  ; vendor identifier
    LOCAL vendorID[13]    :BYTE   ; (function 0)

    LOCAL nameString[48]  :BYTE   ; (function 80000002h-4h, AMD)

    mov   vendorID[12],   0       ; as a precaution
    mov   nameString[47], 0       ; as a precaution

    .IF cpufeat(CPUID) == 0
        print chr$("CPUID not supported"),NL
        jmp   fini
    .ENDIF

    _cpuid 0
    mov   maxCpuidInput,          eax
    mov   DWORD PTR vendorID,     ebx
    mov   DWORD PTR vendorID[4],  edx
    mov   DWORD PTR vendorID[8],  ecx

    SWITCH$ ADDR vendorID    
      CASE$ "GenuineIntel"
          mov   vendor, VENDOR_INTEL
      CASE$ "AuthenticAMD"
          mov   vendor, VENDOR_AMD
      ELSE$
          mov   vendor, VENDOR_OTHER
    ENDSW$

    .IF maxCpuidInput > 0
        _cpuid 1
        mov   esi, eax
        mov   signature,      esi
        mov   steppingID,     brval(esi,3,0)
        mov   model,          brval(esi,7,4)
        mov   family,         brval(esi,11,8)
        mov   extendedModel,  brval(esi,19,16)
        mov   extendedFamily, brval(esi,27,20)
        mov   brandID,        brval(ebx,7,0)

        ; -----------------------------------------------
        ; For AMD only use the family and extendedFamily
        ; returned by function 80000001h, if available.
        ; -----------------------------------------------
        .IF vendor == VENDOR_AMD
            _cpuid 80000000h
            .IF eax > 80000000h
                _cpuid 80000001h
                mov   esi, eax
                mov   family,         brval(esi,11,8)
                mov   extendedFamily, brval(esi,27,20)
            .ENDIF
        .ENDIF
    .ENDIF

    ; -------------------------------------------------------
    ; The AMD Processor Name String, supported starting with
    ; the K5 Model 1, contains the actual processor name.
    ; The Intel Processor Brand String is retrieved in the
    ; same manner as the AMD Processor Name String, but
    ; instead of a processor name it contains frequency and
    ; multiplier information that is of no use here.
    ; -------------------------------------------------------
    .IF vendor == VENDOR_AMD
        _cpuid 80000000h
        mov   maxCpuidInputEx, eax
        .IF eax > 80000000h
            _cpuid 80000001h
            mov   signatureEx, eax
            .IF maxCpuidInputEx >= 80000004h
                _cpuid 80000002h
                mov   DWORD PTR nameString,     eax
                mov   DWORD PTR nameString[4],  ebx
                mov   DWORD PTR nameString[8],  ecx
                mov   DWORD PTR nameString[12], edx
                _cpuid 80000003h
                mov   DWORD PTR nameString[16], eax
                mov   DWORD PTR nameString[20], ebx
                mov   DWORD PTR nameString[24], ecx
                mov   DWORD PTR nameString[28], edx
                _cpuid 80000004h
                mov   DWORD PTR nameString[32], eax
                mov   DWORD PTR nameString[36], ebx
                mov   DWORD PTR nameString[40], ecx
                mov   DWORD PTR nameString[44], edx
            .ENDIF
        .ENDIF
    .ENDIF

    ; ------------------------------------------------------------
    ; Use the Intel rules to calc the displayed family and model.
    ; ------------------------------------------------------------
    m2m   displayFamily, family
    m2m   displayModel, model
    .IF family == 0Fh
        mov   eax, extendedFamily
        add   displayFamily, eax
    .ENDIF
    .IF family == 6 || family == 0Fh
        mov   eax, extendedModel
        add   displayModel, eax
    .ENDIF

    ; -----------------------------------------------------------
    ; Display the family, model, and stepping in decimal instead
    ; of the hex that Intel now recommends.
    ; -----------------------------------------------------------
    print ADDR vendorID
    print chr$("  Family ")
    print ustr$(displayFamily)
    print chr$("  Model ")
    print ustr$(displayModel)
    print chr$("  Stepping ")
    print ustr$(steppingID)
    print chr$(NL)

    ; ----------------------------------------------------------
    ; For Intel processors, display the Intel Brand String from
    ; a table. If the brand ID is zero, display a processor
    ; identification based on the family, model, and features.
    ;
    ; The table is adapted from the IA-32 Intel Architecture
    ; Software Developers Manual Volume 2A, 253666-015,
    ; April 2005, Table 3-19.
    ; ---------------------------------------------------------
    .IF vendor == VENDOR_INTEL
        print chr$("Intel Brand String: ")
        SWITCH brandID
            CASE 0
                print chr$("NA")
                SWITCH family
                    CASE 4
                        print chr$(", processor is 486")
                    CASE 5
                        .IF cpufeat(MMX)
                            print chr$(", processor is Pentium MMX")
                        .ELSE
                            print chr$(", processor is Pentium I")
                        .ENDIF
                    CASE 6
                        .IF cpufeat(SSE)
                            print chr$(", processor is Pentium III")
                        .ELSEIF cpufeat(MMX)
                            print chr$(", processor is Pentium II")
                        .ELSE
                            print chr$(", processor is Pentium Pro")
                    .ENDIF
                ENDSW
                print chr$(NL)
            CASE 1
                print chr$("Intel(R) Celeron(R) processor"),NL
            CASE 2
                print chr$("Intel(R) Pentium(R) III processor"),NL
            CASE 3
                .IF signature == 6B1h
                    print chr$("Intel(R) Celeron(R) processor"),NL
                .ELSE
                    print chr$("Intel(R) Pentium(R) III Xeon(TM) processor"),NL
                .ENDIF
            CASE 4
                print chr$("Intel(R) Pentium(R) III processor"),NL
            CASE 6
                print chr$("Mobile Intel(R) Pentium(R) III processor-M"),NL
            CASE 7
                print chr$("Mobile Intel(R) Celeron(R) processor"),NL
            CASE 8
                print chr$("Intel(R) Pentium(R) 4 processor"),NL
            CASE 9
                print chr$("Intel(R) Pentium(R) 4 processor"),NL
            CASE 0Ah
                print chr$("Intel(R) Celeron(R) processor"),NL
            CASE 0Bh
                .IF signature == 0F13h
                    print chr$("Intel(R) Xeon(TM) processor MP"),NL
                .ELSE
                    print chr$("Intel(R) Xeon(TM) processor"),NL
                .ENDIF
            CASE 0Ch
                print chr$("Intel(R) Xeon(TM) processor MP"),NL
            CASE 0Eh
                .IF signature == 0F13h
                    print chr$("Intel(R) Xeon(TM) processor"),NL
                .ELSE
                    print chr$("Mobile Intel(R) Pentium(R) 4 processor-M"),NL
                .ENDIF
            CASE 0Fh
                print chr$("Mobile Intel(R) Celeron(R) processor"),NL
            CASE 13h
                print chr$("Mobile Intel(R) Celeron(R) processor"),NL
            CASE 16h
                print chr$("Intel(R) Pentium(R) M processor"),NL
            CASE 17h
                print chr$("Mobile Intel(R) Celeron(R) processor"),NL
            DEFAULT
                print chr$("NA, unknown Brand ID"),NL
        ENDSW
    .ENDIF

    ; -----------------------------------------------------------
    ; For AMD processors, display the name string, if available,
    ; or a processor identification based on the family.
    ; -----------------------------------------------------------
    .IF vendor == VENDOR_AMD
        print chr$("AMD Name String: ")
        .IF maxCpuidInputEx >= 80000004h
            print ADDR nameString,NL
        .ELSE
            .IF family == 4
                print chr$("NA, processor is 486/5x86"),NL
            .ELSE
                print chr$("NA, processor is K5"),NL
            .ENDIF
        .ENDIF
    .ENDIF    

    print chr$("Features: ")
    .IF cpufeat(FPU)
        print "FPU"
    .ENDIF
    .IF cpufeat(TSC)
        print " TSC"
    .ENDIF
    .IF cpufeat(CX8)
        print " CX8"
    .ENDIF
    .IF cpufeat(CMOV)
        print " CMOV"
    .ENDIF
    .IF cpufeat(CLFSH)
        print " CLFSH"
    .ENDIF
    .IF cpufeat(FXSR)
        print " FXSR"
    .ENDIF
    .IF cpufeat(HTT)
        print " HTT"
    .ENDIF    
    .IF cpufeat(MMX)
        print " MMX"
    .ENDIF
    .IF cpufeat(SSE)
        print " SSE"
    .ENDIF
    .IF cpufeat(SSE2)
        print " SSE2"
    .ENDIF
    .IF cpufeat(SSE3)
        print " SSE3"
    .ENDIF
    print chr$(NL)

  fini:
    ret
IdentifyCPU endp

